/*
**		Sambar Server JAVA Engine Implementation
**
**		Confidential Property of Tod Sambar
**		(c) Copyright Tod Sambar 2002
**		All rights reserved.
*/
package com.sambar.javaeng;

import java.io.IOException;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.servlet.*;
import javax.servlet.http.*;

/* 
 * Filters
 *
 * Determine and call chains of Filters as configured in web.xml.
 */
public class SambarFilters
{
    private SambarContext	scontext = null;
    private Map 			filterMap = new HashMap();
    private Map 			servletMap = new HashMap();
    private List 			pathFilters = new ArrayList();
    private List 			pathServlets = new ArrayList();

	public SambarFilters(SambarContext scontext)
	{
		this.scontext = scontext;
	}

    SambarServlet newServlet(String name, String className)
    {
        SambarServlet servlet = new SambarServlet(scontext, name, className);
        servletMap.put(name, servlet);
        return servlet;
    }
    
    public SambarServlet getServlet(String name)
    {
        return (SambarServlet)servletMap.get(name);
    }

    public SambarServlet findServlet(String path)
    {
		int nextPathServlet = 0;

        // Find the next applicable path servlet
        while (nextPathServlet < pathServlets.size())
        {
            SambarServlet ss=(SambarServlet)pathServlets.get(nextPathServlet++);
            if (ss.appliesTo(path))
                return ss;
        }

		return null;
    }

    public SambarServlet mapPathToServlet(String pathSpec,
                                        String servletName)
    {
        SambarServlet servlet = (SambarServlet)servletMap.get(servletName);
        if (servlet == null)
            throw new IllegalArgumentException("Unknown servlet :"+servletName);
        
        if (!servlet.isMappedToPath())
            pathServlets.add(servlet);
        servlet.addPathSpec(pathSpec);
        
        return servlet;
    }    

    SambarFilter newFilter(String name, String className)
    {
        SambarFilter filter = new SambarFilter(scontext, name, className);
        filterMap.put(name, filter);
        return filter;
    }
    
    public SambarFilter getFilter(String name)
    {
        return (SambarFilter)filterMap.get(name);
    }

    public SambarFilter mapPathToFilter(String pathSpec,
                                        String filterName)
    {
        SambarFilter filter = (SambarFilter)filterMap.get(filterName);
        if (filter == null)
            throw new IllegalArgumentException("Unknown filter :" + filterName);
        
        if (!filter.isMappedToPath())
            pathFilters.add(filter);
        filter.addPathSpec(pathSpec);
        
        return filter;
    }    

    public synchronized void start()
    {
        // Start servlets
		if (servletMap.size() > 0)
		{
        	Iterator iter = servletMap.values().iterator();
        	while (iter.hasNext())
        	{
            	SambarServlet servlet = (SambarServlet)iter.next();
            	try
				{
					servlet.start();
				}
            	catch(Exception e)
				{
					scontext.log(SambarAPI.LOG_WARN,
     					"Failure starting servlet '" + servlet.getName() + 
						"' : " + e.toString());
				}
        	}
		}

        // Start filters
		if (filterMap.size() > 0)
		{
        	Iterator iter = filterMap.values().iterator();
        	while (iter.hasNext())
        	{
            	SambarFilter filter = (SambarFilter)iter.next();
            	try
				{
					filter.start();
				}
            	catch(Exception e)
				{
					scontext.log(SambarAPI.LOG_WARN,
     					"Failure starting filter '" + filter.getName() + 
						"' : " + e.toString());
				}
        	}
		}
    }
    
    public synchronized void stop()
    {
        // Stop filters
		if (filterMap.size() > 0)
		{
        	Iterator iter = filterMap.values().iterator();
        	while (iter.hasNext())
        	{
            	SambarFilter filter = (SambarFilter)iter.next();
				try
				{
            		filter.stop();
				}
            	catch(Exception e)
				{
					scontext.log(SambarAPI.LOG_WARN,
     					"Failure stopping filter '" + filter.getName() + 
						"' : " + e.toString());
				}
        	}
		}

        // Stop servlets
		if (servletMap.size() > 0)
		{
        	Iterator iter = servletMap.values().iterator();
        	while (iter.hasNext())
        	{
            	SambarServlet servlet = (SambarServlet)iter.next();
				try
				{
            		servlet.stop();
				}
            	catch(Exception e)
				{
					scontext.log(SambarAPI.LOG_WARN,
     					"Failure stopping servlet '" + servlet.getName() + 
						"' : " + e.toString());
				}
        	}
		}
    }
    
    public void handle(String pathInContext,
                       Servlet servlet,
                       ServletRequest req,
                       ServletResponse res)
         throws Exception
    {
		if (filterMap.size() > 0)
		{
			FilterChain chain = new Chain(pathInContext, servlet);
			scontext.log(SambarAPI.LOG_TRACE,
				"SambarFilters chain executing for '" + pathInContext + "'");
			chain.doFilter(req, res);
		}
		else
		{
			servlet.service(req, res);
		}
    }

    private class Chain implements FilterChain
    {
        int nextPathFilter;
        String pathInContext;
        List servletFilters;
		Servlet servlet;
        
        Chain(String pathInContext, Servlet servlet)
        {
            nextPathFilter=0;
            this.pathInContext=pathInContext;
			this.servlet=servlet;
        }
        
        public void doFilter(ServletRequest request, ServletResponse response)
            throws IOException, ServletException
        {
            // Find the next applicable path filter
            while (nextPathFilter < pathFilters.size())
            {
                SambarFilter sf=(SambarFilter)pathFilters.get(nextPathFilter++);
                if (sf.appliesTo(pathInContext))
                {
                    Filter filter = sf.getFilter();
                    filter.doFilter(request,response,this);
                    return;
                }
            }

			if (servlet != null)
				servlet.service(request, response);
        }   
    }
}
