/*
**		Sambar Server JAVA Engine Implementation
**
**		Confidential Property of Tod Sambar
**		(c) Copyright Tod Sambar 2002
**		All rights reserved.
*/
package com.sambar.javaeng;

import java.io.IOException;

import java.util.ArrayList;
import java.util.EventListener;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.servlet.*;
import javax.servlet.http.*;

/* 
 * Listeners
 *
 * Determine and call chains of Listeners as configured in web.xml.
 */
public class SambarListeners
{
    private SambarContext	scontext = null;
    private ArrayList 		contextListeners = null;
    private ArrayList 		contextAttributeListeners = null;
    private ArrayList		sessionListeners = null;
    private ArrayList		sessionAttributeListeners = null;

	public SambarListeners(SambarContext scontext)
	{
		this.scontext = scontext;
	}

    public void initListener(String name)
    {
		Object listener = null; 
		try
		{
			Class listenerClass = scontext.getClassLoader().loadClass(name);
			listener = listenerClass.newInstance();
		}
		catch (Exception e)
		{
			scontext.log(SambarAPI.LOG_ERROR,
				"SambarListeners failed to load listener '" + name + "'");
			return;
		}

		if (!(listener instanceof EventListener))
		{
			scontext.log(SambarAPI.LOG_ERROR,
				"Listener '" + name + "' is not an EventListener.");
			return;
		}

		if (listener instanceof ServletContextListener)
		{
			if (contextListeners == null)
				contextListeners = new ArrayList(3);
			contextListeners.add(listener);
		}
		else if (listener instanceof ServletContextAttributeListener)
		{
			if (contextAttributeListeners == null)
				contextAttributeListeners = new ArrayList(3);
			contextAttributeListeners.add(listener);
		}
		else if (listener instanceof HttpSessionListener)
		{
			if (sessionListeners == null)
				sessionListeners = new ArrayList(3);
			sessionListeners.add(listener);
		}
		else
		{
			scontext.log(SambarAPI.LOG_ERROR,
				"Listener '" + name + "' is not a supported listener type.");
			return;
		}
    }

    public synchronized void setAttribute(String name, Object value)
    {
		if (contextAttributeListeners != null)
		{
			Object old = scontext.getAttribute(name);

			ServletContextAttributeEvent event = 
				new ServletContextAttributeEvent(scontext, name, 
				old != null ? old : value);

        	for (int i = 0; i < contextAttributeListeners.size(); i++)
			{
				try
				{
					ServletContextAttributeListener l = 
						(ServletContextAttributeListener)
						contextAttributeListeners.get(i);
					if (old == null)
					{
						l.attributeAdded(event);
					}
					else if (value == null)
					{
						l.attributeRemoved(event);
					}
					else 
					{
						l.attributeReplaced(event);
					}
				}
            	catch(Exception e)
				{
					scontext.log(SambarAPI.LOG_ERROR,
     					"Failure setting context attribute '" + name + 
						"' : " + e.toString());
				}
        	}
		}
	}

    public synchronized void start()
    {
		if (contextListeners != null)
		{
			ServletContextEvent event = new ServletContextEvent(scontext);
        	for (int i = 0; i < contextListeners.size(); i++)
			{
				try
				{
					((ServletContextListener)contextListeners.get(i))
						.contextInitialized(event);
				}
            	catch(Exception e)
				{
					scontext.log(SambarAPI.LOG_ERROR,
     					"Failure starting context listener : " + e.toString());
				}
        	}
		}
    }
    
    public synchronized void stop()
    {
		if (contextListeners != null)
		{
			ServletContextEvent event = new ServletContextEvent(scontext);
        	for (int i = contextListeners.size(); i-- > 0;)
			{
				try
				{
					((ServletContextListener)contextListeners.get(i))
						.contextDestroyed(event);
				}
            	catch(Exception e)
				{
					scontext.log(SambarAPI.LOG_ERROR,
     					"Failure starting context listener : " + e.toString());
				}
        	}

			contextListeners.clear();
		}

		if (contextAttributeListeners != null)
			contextAttributeListeners.clear();
    }

    public void sessionStart(HttpSession session)
    {
		if (sessionListeners != null)
		{
			HttpSessionEvent event = new HttpSessionEvent(session);
        	for (int i = 0; i < sessionListeners.size(); i++)
			{
				try
				{
					((HttpSessionListener)sessionListeners.get(i))
						.sessionCreated(event);
				}
            	catch(Exception e)
				{
					scontext.log(SambarAPI.LOG_ERROR,
     					"Failure calling session created listener : " + 
						e.toString());
				}
        	}
		}
    }

    public void sessionStop(HttpSession session)
    {
		if (sessionListeners != null)
		{
			HttpSessionEvent event = new HttpSessionEvent(session);
        	for (int i = 0; i < sessionListeners.size(); i++)
			{
				try
				{
					((HttpSessionListener)sessionListeners.get(i))
						.sessionDestroyed(event);
				}
            	catch(Exception e)
				{
					scontext.log(SambarAPI.LOG_ERROR,
     					"Failure calling session destroy listener : " + 
						e.toString());
				}
        	}
		}
    }
}
