/*
**	Sambar Server JAVA Engine Implementation
**
**	Confidential Property of Tod Sambar
**	(c) Copyright Tod Sambar 2002
**	All rights reserved.
*/
package com.sambar.javaeng;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Collections;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Map;
import javax.servlet.*;

public class SambarServlet
{
    protected Servlet 			servlet = null;
    protected Config 			config = null;
    protected Map 				initParams = null;
	protected boolean			loadOnStartup = false;
    protected HashMap 			pathSpecs = null;
	protected String			name;
	protected String			classname;
	protected SambarContext  	scontext;
    
    /* ---------------------------------------------------------------- */
    public SambarServlet(SambarContext scontext, String name, String className)
    {
        if ((name == null) || (name.length() == 0))
		{
            throw new IllegalArgumentException("SambarServlet: No name for " + 
				className);
		}

        if ((className == null) || (className.length() == 0))
		{
            throw new IllegalArgumentException(
				"SambarServlet: No classname for " + name);
		}

		this.name = name;
		this.classname = className;
		this.scontext = scontext;
    }

	public String getName()
	{
		return name;
	}

	public String getClassName()
	{
		return classname;
	}

    public void addPathSpec(String pathSpec)
    {
		if ((pathSpec == null) || (pathSpec.length() == 0))
			return;

        if (pathSpecs == null)
            pathSpecs = new HashMap();

		// Strip off leading slash to deal with how servlets get mapped into
		// the server.
		if (pathSpec.charAt(0) == '/')
			pathSpec = pathSpec.substring(1);

        pathSpecs.put(pathSpec, pathSpec);
    }
    
    public boolean isMappedToPath()
    {
        return (pathSpecs != null);
    }

    public void setLoadOnStartup(boolean load)
    {
        loadOnStartup = load;
    }
    
    public boolean appliesTo(String path)
    {
		if (pathSpecs == null)
		{
			scontext.log("pathspecs == null");
			return (false);
		}

		if (path == null)
		{
			scontext.log("path == null");
            return (false);
		}

		Iterator iter = pathSpecs.values().iterator();
		while (iter.hasNext())
		{
			String pattern = (String)iter.next();
			
        	// Case 1 - Exact Match
        	if (pattern.equals(path))
            	return (true);

			// Case 1.1 - Starts With pattern followed by a slash (pathInfo)
        	if ((path.startsWith(pattern)) && 
				(path.charAt(pattern.length()) == '/'))
			{
            	return (true);
			}

        	// Case 2 - Path Match ("/.../*")
        	if ((pattern.equals("/*")) || (pattern.equals("*")))
            	return (true);      // Optimize a common case

	        if (pattern.endsWith("/*")) 
			{
           		String comparePath = path;
            	while (true) 
				{
               		if (pattern.equals(comparePath + "/*"))
                    	return (true);
                	int slash = comparePath.lastIndexOf('/');
                	if (slash < 0)
                   		break;
                	comparePath = comparePath.substring(0, slash);
            	}

            	continue;
        	}

        	// Case 3 - Extension Match
        	if (pattern.startsWith("*.")) 
			{
            	int slash = path.lastIndexOf('/');
            	int period = path.lastIndexOf('.');
            	if ((slash >= 0) && (period > slash))
				{
                	if (pattern.equals("*." + path.substring(period + 1)))
						return (true);
				}
        	}
        }

        return (false);
    }

    public String getPathInfo(String path)
    {
		if ((pathSpecs == null) || (path == null))
			return (null);

		Iterator iter = pathSpecs.values().iterator();
		while (iter.hasNext())
		{
			String pattern = (String)iter.next();
			
        	// Exact Match
        	if (pattern.equals(path))
            	return (null);

			// Starts With pattern followed by a slash (pathInfo)
        	if ((path.startsWith(pattern)) && 
				(path.charAt(pattern.length()) == '/'))
			{
            	return (path.substring(pattern.length()));
			}

            if (pattern.endsWith("/*"))
            {
                String comparePath = path;
                while (true)
                {
                    if (pattern.equals(comparePath + "/*"))
						return path.substring(comparePath.length());

                    int slash = comparePath.lastIndexOf('/');
                    if (slash < 0)
                        break;
                    comparePath = comparePath.substring(0, slash);
                }
            }
		}

		return (null);
	}

    public void setInitParameter(String param, String value)
    {
        if (initParams == null)
            initParams = new HashMap(3);
        initParams.put(name, value);
    }

    public String getInitParameter(String param)
    {
        if (initParams == null)
            return null;
        return (String)initParams.get(param);
    }

    public Enumeration getInitParameterNames()
    {
        if (initParams == null)
            return Collections.enumeration(Collections.EMPTY_LIST);
        return Collections.enumeration(initParams.keySet());
    }

    public void start() throws Exception
    {
		if (loadOnStartup)
		{
			scontext.log("Initialzing servlet '" + name + "'");
			ClassLoader loader = scontext.getClassLoader();

        	config = new Config();
			Servlet servlet = scontext.getServlet(null, classname, 
				config, false);
        	if (servlet == null)
        	{
            	throw new IllegalStateException("Servlet class " + classname +
					" was not found.");
        	}
		}
    }

    public void stop()
    {
        if (servlet != null)
		{
            servlet.destroy();
        	config = null;
        	servlet = null;
		}
    }
    
    class Config implements ServletConfig
    {
        public String getServletName()
        {
            return SambarServlet.this.name;
        }

        public ServletContext getServletContext()
        {
            return SambarServlet.this.scontext;
        }
        
        public String getInitParameter(String param)
        {
            return SambarServlet.this.getInitParameter(param);
        }
    
        public Enumeration getInitParameterNames()
        {
            return SambarServlet.this.getInitParameterNames();
        }
    }
}
